import uid from 'uid-safe';
import { isString } from 'lodash';
import { serialize } from 'cookie';
class SvelteKitSession {
    sessionData;
    sessionId;
    sessionName;
    sessionManager;
    requestEvent;
    constructor(data, sessionId, sessionName, sessionManager, requestEvent) {
        this.sessionData = data;
        this.sessionId = sessionId;
        this.sessionName = sessionName;
        this.sessionManager = sessionManager;
        this.requestEvent = requestEvent;
        return new Proxy(this, {
            set(target, name, value) {
                if (name === 'data') {
                    target.sessionData = value;
                    return true;
                }
                target.sessionData[name] = value;
                return true;
            },
            get(target, name) {
                if (typeof target[name] !== 'function') {
                    if (name === 'data') {
                        return target.sessionData;
                    }
                    else if (name === 'id') {
                        return target.sessionId;
                    }
                    return target.sessionData[name];
                }
                else if (name === 'destroy') {
                    return target.destroy.bind(target);
                }
                throw new Error(`Session.${String(name)} is not a property`);
            }
        });
    }
    get data() {
        return this.sessionData;
    }
    set data(value) { }
    get id() {
        return this.sessionId;
    }
    async destroy() {
        await this.sessionManager.deleteSession(this.id, this.requestEvent);
    }
}
export class SessionManager {
    /**
     * @param sessionOptions - Session options
     * @param store - compatible store
     * @param cookieOptions - cookie options
     */
    store;
    ttl;
    name;
    cookieOptions;
    refreshSession;
    constructor(sessionOptions, store, cookieOptions) {
        this.store = store;
        this.ttl = sessionOptions.ttl;
        this.name = sessionOptions.name ?? 'sessionId';
        this.refreshSession = sessionOptions.refreshSession ?? true;
        this.cookieOptions = {
            ...{
                maxAge: this.ttl
            },
            ...cookieOptions
        };
    }
    /**
     *
     * @param cookieOptions
     * @returns override cookie options for this request
     */
    getCookieOptions(cookieOptions) {
        return {
            ...this.cookieOptions,
            expires: new Date(Date.now() + this.ttl * 1000),
            ...cookieOptions
        };
    }
    /**
     * **!!!!** - Use `Session.destroy()` if you want to delete the session from the store and destroy the cookie. - **!!!!**
     *
     * Deletes the session from the store
     *
     * Only pass in `sessionId`
     * @param sessionId - session id
     * @param requestEvent - !! Don't pass this !!
     */
    async deleteSession(sessionId, requestEvent) {
        await this.store.delete(sessionId);
        if (requestEvent) {
            delete requestEvent.locals.session;
            requestEvent.cookies.delete(this.name, this.getCookieOptions());
        }
    }
    async createSession(session, requestEvent) {
        const sessionId = await uid(24);
        await this.store.set(sessionId, session, this.ttl);
        requestEvent.cookies.set(this.name, sessionId, this.getCookieOptions());
        requestEvent.locals.session = new SvelteKitSession(session, sessionId, this.name, this, requestEvent);
        return requestEvent.locals.session;
    }
    async getSession(cookiesOrSessionId) {
        let sessionId;
        if (isString(cookiesOrSessionId)) {
            sessionId = cookiesOrSessionId;
        }
        else {
            sessionId = cookiesOrSessionId.get(this.name);
            if (!sessionId)
                return null;
        }
        return await this.store.get(sessionId);
    }
    async updateSession(sessionId, sessionData) {
        await this.store.set(sessionId, sessionData);
        if (this.refreshSession) {
            await this.store.set(sessionId, sessionData, this.ttl);
        }
        else {
            await this.store.set(sessionId, sessionData);
        }
    }
    async handleSession(requestEvent) {
        delete requestEvent.locals.session;
        const cookie = requestEvent.cookies.get(this.name);
        if (!cookie) {
            return;
        }
        const session = await this.getSession(cookie);
        if (!session) {
            requestEvent.cookies.delete(this.name, this.getCookieOptions());
            return;
        }
        requestEvent.locals.session = new SvelteKitSession(session, cookie, this.name, this, requestEvent);
    }
    async resolve(requestEvent, resolve) {
        const response = await resolve(requestEvent);
        const session = requestEvent.locals.session;
        const sessionId = requestEvent.cookies.get(this.name);
        if (session && sessionId) {
            await this.updateSession(sessionId, session.data);
            response.headers.append('Set-Cookie', serialize(this.name, sessionId, this.getCookieOptions()));
        }
        return response;
    }
    async listSessions() {
        return await this.store.keys();
    }
    async deleteAllSessions() {
        this.store.clear();
    }
}
export function handleSession(sessionManager) {
    return async function handle({ event, resolve }) {
        // @ts-expect-error - handleSession is private
        await sessionManager.handleSession(event);
        // @ts-expect-error - resolve is private
        const response = await sessionManager.resolve(event, resolve);
        return response;
    };
}
export default SessionManager;
